#' Symbolic Sum of All the Elements
#'
#' @param obj The object whose elements are to be summed.
#'
#' @details
#' This is equivalent to \code{Reduce( Plus, obj )}.
#'
#' @examples
#' A <- demomat(3,3,root="a")
#' Sum(A)
#' Sum( diag(A) )
#' Sum( diag(matTimes(t(A),A)) )
#'
#' @return
#' The symbolic sum of obj.
#'
#' @export
#'

Sum <- function( obj ){
 # sum of all the elements of obj
 # Shin-ichi Mayekawa
 # 20150907
 #
 res=Reduce( Plus, obj )
 return( res )

} # end of Sum



#' Symbolic Product of All the Elements
#'
#' @param obj The object whose elements are to be multiplied.
#'
#' @details
#' This is equivalent to \code{Reduce( Times, obj )}.
#'
#' @examples
#' A <- demomat(3,3,root="a"); A[2,2]="5-a22"
#' Prod(A)
#' Prod( diag(A) )
#'
#' @return
#' The symbolic product of obj.
#'
#' @export
#'

Prod <- function( obj ){
 # product of all the elements of obj
 # Shin-ichi Mayekawa
 # 20150907
 #
 res=Reduce( Times, obj )
 return( res )

} # end of Prod



