#' trace of the input matrix
#'
#' @param A Input square matrix or scalar
#' @return trace of A
#' @details
#' Can be used with a character matrix.
#'
#' @examples
#' tr(demomat(2,2))
#'
#' # Semi Symbolic Partitioned Matrix:  Super Matrix with SMdim attributes
#' A=demomat(c(2,3),c(2,3),root="a")
#' Printb(A)
#' tr(A)
#'
#' # Fully Symbolic Partitioned Matrix: No SMdim attribute but
#' # fullsymb attribute
#' Ap=demomat(2,2,root="A", fullsymb=1)
#' Printb(Ap)
#' tr(Ap)
#'
#' @export
#'

tr <- function(A){
 ## trace
 ## Shin-ichi Mayekawa
 ## 111109
 ## is.matrix: 20150531cot
 # character input: 20150908
 # vectorize 1 x 1 matrix Sum(diag(A)): 20160106
 # fullsymb attribute: 20200527
 #

 # fullsymb attribute
 fullsymb=attributes(A)$fullsymb

 if( !is.null(fullsymb) ){
   res=Sum( map("tr", diag(A)))

   #   1        2         3     4           5      6
   #   (top | *)( tr(t(  )(sign)( variable )( ))  )($ or */ ) ->
   v3="(^|[-+*`~])(tr\\(t\\()([-+]*)([[:alpha:]][0-9_]*)(\\)\\))($|[-+*/`~])"
   res=gsub( v3, "\\1tr(\\3\\4)\\6", res )

   return( res )
 }

 if( is.matrix(A) ){
  if( nrow(A) == ncol(A) ){
   if( is.numeric(A) ) return( sum(diag(A)) )
   else return( c(Sum(diag(A))) )
  }
  else{
   cat("\nerror1:(tr) Input matrix is not square.\n")
   return()
  }
 }
 else if( is.vector(A) & length(A) == 1 ) return(A)
 else{
  cat("\nerror1:(tr) Input is not a matrix.\n")
  return()
 }

} # end of tr



