######################################################################
################## Special Operators #################################
######################################################################


#' Plus operator for Elementwise Sum
#' @usage mat1 \%p\% mat2
#' @param mat1 A matrix
#' @param mat2 A matrix of the same size as mat1
#' @details Plus function \cr
#' \strong{Note that symbolic operators have equal precedence. Use ( ).}
#' @export
#'
`%p%` <- Plus


#' Minus operator for Elementwise Difference
#' @usage mat1 \%m\% mat2
#' @param mat1 A matrix
#' @param mat2 A matrix of the same size as mat1
#' @details Minus function \cr
#' \strong{Note that symbolic operators have equal precedence. Use ( ).}
#' @export
#'
`%m%` <- Minus


#' Times operator
#' @usage mat1 \%t\% mat2
#' @param mat1 A matrix or vector
#' @param mat2 A matrix or vector
#' @details
#' Times function. \cr\cr
#' \%T\% is for matrix multiplication with matTimes function. \cr\cr
#' \strong{Note that symbolic operators have equal precedence. Use ( ).}
#' @export
#'
`%t%` <- Times


#' matTimes operator for Matrix Product
#' @usage mat1 \%T\% mat2
#' @param mat1 A matrix
#' @param mat2 A matrix satisfying ncol(mat1) == nrow(mat2)
#' @details
#' matTimes function \cr\cr
#' \%t\% is for elementwise multiplication with Times function. \cr\cr
#' \strong{Note that symbolic operators have equal precedence. Use ( ).}
#' @export
#'
`%T%` <- matTimes


#' matKTimes operator for Kronecker Product
#' @usage mat1 \%K\% mat2
#' @param mat1 A matrix
#' @param mat2 A matrix)
#' @details matKTimes function \cr
#' \strong{Note that symbolic operators have equal precedence. Use ( ).}
#' @export
#'
`%@%` <- matKTimes


#' Mathematica like postfix replacement
#' @param exp argument of the function
#' @param values replacement of the form "pat11=pat21; pat12=pat22"
#' @details
#' Equivalent to matReplace( exp, values=values ).
#' @export
#'
`%/.%` <- function( exp, values ){
 # Postfix Replacement
 # Shin-ichi Mayekawa
 # 20150924
 #

 matReplace( exp, values=values )
}


######################################################################
################## Special Operators #################################
######################################################################



