% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Det.R
\name{Det}
\alias{Det}
\title{Determinant of a Symbolic Matrix by Sweeping}
\usage{
Det(A, loc = 1:nrow(A), log = 0, simplify = 0, debug = 0)
}
\arguments{
\item{A}{A square matrix}

\item{loc}{vector of pivot locations}

\item{log}{= 1 to calculate log determinant}

\item{simplify}{= 0 not to use Simplify nor remove_paren \cr
= 1 to use Simplify}

\item{debug}{= 1 to print details}
}
\value{
The determinant of A
}
\description{
Determinant of a Symbolic Matrix by Sweeping
}
\details{
Definition of Sweep Operator\cr
C <- matSweep( A, p ) means: \cr
 c[i,j] <- c[i,j] - c[i,p]*c[p,j]/c[p,p] \cr
 c[p,] <- c[p,]/c[p,p] \cr
 c[,p] <- - c[,p]/c[p,p] \cr

# Inversion of A.  \cr
matSweep( A, 1:ncol(A) ) == solve(A) \cr
This is equivalent to Inv(A).
\cr\cr
The determinant of A is given as the product of all the pivots, c[p,p],
when inverting A using sweep operator.
}
\examples{

# determinant of 2 x 2 A
A <- demomat(2,2,root="a")
Det(A)
Det(A,log=1)

# n x n ( slow !)
set.seed(1701)
n=3
B <- demomat(n,n,root="b")
dd <- Det(B)
Bn <- matrix(runif(n*n),n)
Print(Eval(dd,B=Bn),det(Bn))

# Fully Symbolic Partitioned Matrix: No SMdim attribute but
# fullsymb attribute
Ap=demomat(2,2,root="A", fullsymb=1)
Print(Ap)
Det(Ap)
Det(Ap, 2:1)

}
