% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DetL.R
\name{DetL}
\alias{DetL}
\title{Determinant of a Symbolic Matrix by Laplace Expansion}
\usage{
DetL(A, row = 1, col = NULL, maxlevel = 1, level = 0, simplify = 0, debug = 0)
}
\arguments{
\item{A}{A square matrix (numeric or symbolic)}

\item{row}{row number for Laplace Expansion}

\item{col}{col number for Laplace Expansion}

\item{maxlevel}{depth of recursive calls}

\item{level}{Do Not Modify This.}

\item{simplify}{= 0 not to use Simplify nor remove_paren \cr
= 1 to use Simplify}

\item{debug}{= 1 to print details}
}
\value{
The determinant of A
}
\description{
Determinant of a Symbolic Matrix by Laplace Expansion
}
\details{
THe Laplace Expansion is defined as:
\preformatted{
 DetL(A) = \sum_{j=1}^n  A[i,j] x cofactor(A,i,j)
 cofactor(A,i,j) = (-1)^{i+j} DetL( A[-i,-j] )
}
where DetL( A[-i,-j] ) is called the minor M(i,j) of A.
}
\examples{

# determinant of 2 x 2 matrix
Aa2 <- demomat(2,2,root="a")
DetL(Aa2)

# Triangular matrix
L=demomat(4,4,root="L", shape="lowert")
DetL(L, maxlevel=3)
U=demomat(4,4,root="U", shape="uppert")
DetL(U, maxlevel=3)

# 3 x 3 matrix
Aa3 <- matrix(letters[1:9],3,3,byrow=1)
# With 2nd order minors
DetL(Aa3)
DetL(Aa3, row=2)

# full Laplace Expansion
( d1 <- DetL(Aa3, maxlevel=2) )
( d2 <- DetL(Aa3, row=2, maxlevel=2) )
( d3 <- DetL(Aa3, row=NULL, col=1, maxlevel=2) )

# Numerical verification
set.seed(1701)
An <- matrix( runif(9), 3, 3 )
Printm(det(An))
Printm( Eval(d1, Aa3=An), Eval(d2, Aa3=An), Eval(d3, Aa3=An) )

# Comparison with Det function using MatSwp.
( d4 <- Det(Aa3) )
Printm( Eval(d4, Aa3=An) )


# 4 x 4 matrix
Aa4 <- matrix(letters[1:16],4,4,byrow=1)
DetL(Aa4)
# Below, A2 stands for a submatrix of the first level.
# Note that not all the A2 are the same!!
DetL(Aa4, maxlevel=2)
DetL(Aa4, maxlevel=3)

# General n x n case. (When n is large, it is slow.)
n <- 4
( B <- demomat(n,n,root="b") )
( dd <- DetL(B, maxlevel=n) )

# Numerical verification
set.seed(1701)
Bn <- matrix( runif(n*n) ,n )
Printm( Eval(dd,B=Bn), det(Bn) )

}
