% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Expand.R
\name{Expand}
\alias{Expand}
\title{Expand Expressions by Distributing a Multiplicative Factor}
\usage{
Expand(expr, all = 0, method = 1:2, simplify = 1, verify = 1, debug = 0)
}
\arguments{
\item{expr}{A vector or list of symbolic expressions}

\item{all}{= 1 to apply Expand repeatedly}

\item{method}{= 1 to expand a*(...) \cr
= 2 to expand (...)*a}

\item{simplify}{= 0 to skip the simplification of the result}

\item{verify}{= 0 not to check the result by \code{is_valid}}

\item{debug}{= 1 to print intermediate result}
}
\value{
Expanded expression
}
\description{
Expand Expressions by Distributing a Multiplicative Factor
}
\details{
This function expands a single occurence of \cr
\code{a*(b+c)} and \code{(a+b)*c} \cr
to \cr
 \code{a*b+a*c}  and \code{a*c+b*c}, resp., \cr
where \code{a} is a single multiplicative factor and
\code{b} and \code{c} are additive terms. \cr\cr
Matrix multiplications such as \%*\% and \%@\% can be used. \cr
\cr
Repeated use may expand it further.
}
\examples{
Expand(c("a*(b+c)","-(b+c)*a"))
Expand(c("a/(b*c)","-(b*c)/a"))

Expand("(a+b)*(F(x)-G(y))") |> print() |> Expand()

Expand("-a*(b-c)*(d+e)") |> print() |> Expand()

Expand( "((+a*(b-c*d)*e+f)*g)*h - (((+a*b-c*d)*e+f)*g)*h" ) |>
print() |> Expand()

Expand("-t(A)\%*\%(B-C)\%*\%D")

Expand("-inv(A)\%*\%(A-B)")
Expand("-inv(A)\%*\%(A-B)", simplify=0)

Expand("-(A-B)\%*\%(C-D)+E")
Expand("-(A-B)\%*\%(C-D)+E", all=1)

}
