% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Simplify.R
\name{Simplify}
\alias{Simplify}
\title{Simplify Symbolic Expressions
(very primitive version)}
\usage{
Simplify(
  expr,
  all = 0,
  method = 1:3,
  func_level = 0,
  expandfuncarg = 1,
  markso = 1,
  verify = 1,
  debug = 0
)
}
\arguments{
\item{expr}{Expression (vector or matrix)}

\item{all}{= 1 to apply Simplify repeatedly.}

\item{method}{How to simplify: default method is 1:3. \cr
= 1 : varname*num -> num*varname \cr
= 2 : processing 1s and 0s, a/a -> 1, etc
w/o using \code{find_terms} \cr
= 3 : a-a or -a+a -> 0  or  0*func(x) -> 0 \cr
= 4 : to print the result of analyze_expr( expr )}

\item{func_level}{Level of function argument simplification \cr
= 0 not to simplify function arguments \cr
= 1 to simplify the top level function arguments \cr
= 2 to simplify the 2nd level function arguments
(the arguments of the functions in functions.)}

\item{expandfuncarg}{= 0 not to expand the function argument. \cr
= 1 to expand function argument with function name \cr
= 2 to expand function argument (body)}

\item{markso}{= 0 not to mark the special opeartors}

\item{verify}{= 0 not to check the result by \code{is_valid}}

\item{debug}{= 1 to print details}
}
\value{
Simplified expression
}
\description{
Simplify Symbolic Expressions
(very primitive version)
}
\details{
This is a very primitive version. \cr
This function applies some mathematical rules such as \cr
\code{0*a -> 0} or \code{a*1 -> a} described as regular expressions
to an input expression. \cr\cr
In method=1 and 2, \code{gsub} function is used to replace
the patterns such as \code{0*a} or \code{-((a))} with their simplified
mathematically equivalent forms. \cr
In method=3, input expression is decomposed into terms and the patterns
such as \code{func(a)*0} or \code{X\%*\%inv(X)} will be simplified.
\cr
Can handle fully symbolic matrix expressions.
\cr
\cr
Repeated use of this function may simplify the result further. \cr
Simplify may work better if you \code{Expand} the expression
prior to submission. \cr
\cr
When called from remove_paren, use method=1:3. \cr
}
\examples{
expr="-((a))-b/b+0*c+d*0+1*e-e*2+f-3*f+0/(g+h) - (x-y)*(((z+1)))"
Simplify( expr, all=1 ) |> print() |> Expand(all=1)

( res <- matTimes( demomat(2,2,root="a"), demomat(2,1), simplify=0 ) )
Simplify(res)

Simplify(c("1*b","1*(b)","1*f(b)"))
Simplify(c("0*b","0*(b)","0*f(b)"))

dd <- Det(demomat(2,2,root="a"))
Simplify(Expand(dd))

expr1=c("-(X+A)\%*\%inv(X+A)\%*\%(X+A)\%*\%inv(X+A)"
   , "-(X+A1)\%*\%inv(X+A)\%*\%(X+A)\%*\%inv(X+A)")
Simplify(expr1)
Simplify(expr1, all=1)

( res <- Simplify( "a+b+2*a+2*b-3*a+3*b" ) )
Simplify(res)
# Sorry.

# semi symbolic matrix
A=demomat(2,2,root="a")
AinvA=matTimes( A, Inv(A) )
Simplify( AinvA, all=1 )
Simplify( Expand(AinvA, all=1), all=1 )
# Sorry, cannot simplify \code{AinvA[2,1]} and \code{AinvA[2,2]}.

# fully symbolic matrix
A=demomat(2,2,root="A", fullsymb=1)
A1=matSweep(A,1)
A11=matSweep(A1,1)
A11s=Simplify(A11)
Expand(A11s)


}
