% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze_expr.R
\name{analyze_expr}
\alias{analyze_expr}
\title{Decompose an Expression into Additive Terms and Multiplicative Factors}
\usage{
analyze_expr(expr, sort = 0, print = 0, debug = 0)
}
\arguments{
\item{expr}{An input expression}

\item{sort}{= 1 to sort the result by variable name}

\item{print}{= 0 not to print the result}

\item{debug}{= 1 to print intermediate info}
}
\value{
A list of tab, valid and identical, where \cr
 tab is the table shown above, \cr
 valid is the flag to see if the expression is valid, \cr
 identical is the flag to see if the input and the output are identical.
}
\description{
Decompose an Expression into Additive Terms and Multiplicative Factors
}
\details{
The result of the analysis is stored in a character matrix named \code{tab}
 .  \cr
 If the input expression is \cr
  \code{"-inv(A)+(B+C)+D\%*\%E+F\%*\%(G-(H)-(I)+J)"} \cr
 it will be decomposed into the terms and factors in the matrix shown below:
  \preformatted{
       topers   o1  factor_1    o2     factor_2
  -----------------------------------------------------
  term_1    -       inv(A)
  term_2    +       (B+C)
  term_3    +       D           \%*\%    E
  term_4    +       F           \%*\%    (G-(H)-(I)+J)
}


\cr
  Each row of the tab matrix corresponds to a term in \code{expr} and
  each term is decomposed into factors and operators. \cr

  \code{topers} and \code{o1} are the operators to be attached to factor1.\cr
  (\code{o1} may be an empty string vector as shown in the example above.)\cr
  \code{o2} is the operator to be attached to \code{factor2}, etc. \cr

  \code{topers} and \code{o1} are additive operators, + or -
  ,  and \code{op2, op3,...}, are multiplicative operators, *, \%*\%, etc.

 From this, the original expression can be recovered as:
 \preformatted{
  paste( apply(tab,1, function(x){paste(x,collapse="")} ), collapse="")
  }
}
\examples{
analyze_expr( "-inv(A)+(B+C)+D\%*\%E+F\%*\%(G-(H)-(I)+J)" )
analyze_expr( "-inv(A)+(B+C)+D\%*\%E+F\%*\%(G-(H)-(I)+J)", sort=1 )

# recovery
expr <- "-inv(A)+2*(B-C)+Psi(X)"
tab <- analyze_expr( expr )$tab
expr2 <- paste(apply(tab,1, function(x){paste(x,collapse="")}), collapse="")
identical(expr,expr2)

# scalar expressions
set.seed(1702)

nexpr=20
for( i in 1:nexpr ){

 # generate expression
 natom=10
 expr=gen_random_expr( natom=natom, nparen=4, oper=c("*","+","-") )
 if( runif(1) < 0.2 ) expr=sub( "a", "F(a)", expr )
 if( runif(1) < 0.3 ) expr=sub( "d", "F1(d)", expr )
 if( runif(1) < 0.2 ) expr=sub( letters[natom], "Ff2(x)", expr )

 # analyze expression
 res=analyze_expr( expr, print=1 )

}


# matrix expressions
set.seed(1702)

nexpr=20
for( i in 1:nexpr ){

  # generate expression
  natom=10
  expr=gen_random_expr( natom=natom, nparen=4
                        , alpha=LETTERS[1:natom], oper=c("\%*\%","+","-") )
  if( runif(1) < 0.3 ) expr=sub( "A", "inv(A)", expr )
  if( runif(1) < 0.3 ) expr=sub( "C", "diag(C)", expr )
  if( runif(1) < 0.3 ) expr=sub( LETTERS[natom], "ginv(X)", expr )

  # analyze expression
  res=analyze_expr( expr, print=1 )

}

}
