% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_a_term.R
\name{find_a_term}
\alias{find_a_term}
\title{Find a term separated by a multiplicative operator}
\usage{
find_a_term(expr, pattern = NULL, complete = 1)
}
\arguments{
\item{expr}{A string containing an expression}

\item{pattern}{A pattern to be used to identify a variable \cr
The default pattern is \code{"([[:digit:]]+|[[:alpha:]][[:alnum:]_]*)"}.}

\item{complete}{= 0 to find the first variable, not the first term}
}
\value{
If complete=1, a list of \cr
\preformatted{
  term  The term found w/o trailing and preceeding signs.
  loc   The starting location of the term in expr.
  len   The length of the term
  pre   The preceeding string to the left of the term, if any.
  post  The trailing string to the right of the term, if any.
}

\code{term} will be a valid expression but
depending on the input, \code{pre} and \code{post} may not be
valid expressions.

From \code{expr} the term can be obtained as \cr
\preformatted{
res <- find_a_term( expr )
term <- substr(expr,res$loc,res$loc+res$len-1)
}
See the example below.
}
\description{
Find a term separated by a multiplicative operator
}
\details{
This function finds the first term of expr.
The term is defined as the product (including division) of variables.
A variable is defined as the string consisting of valid root characters
trailed by subscripts in [ and ], if any.
Those characters in a balanced parens can be regarded as a variable
as well as a function reference such as f(args).
Therefore, \code{expr} such as \code{A[a,b]} or \code{f(args)}
 are treated equally.

When \code{complete=0}, the function returns the first variable.
WHen \code{complete=1}, the function returns the first term.

This function searches for a variable in \code{expr} from left to right.
If a variable is found and it is trailed by a multiplicative operator,
the search continues to the right again.
}
\examples{
find_a_term( "-a*b+c")
find_a_term( "-f(a*b)+c")
find_a_term( "a*F(b+c)")

find_a_term( "-a*b+c", complete=0)

expr="-b[i,j]*a+c(a,b))-f(x)+a2"
res=find_a_term( expr )
Print(res)
Print(substr(expr,res$loc,res$loc+res$len-1), res$term)

}
