% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_factors.R
\name{find_factors}
\alias{find_factors}
\title{Decompose a Term into Multiplicative Factors separated by
\code{ *, /,  \%*\%,  \%\@\%}.}
\usage{
find_factors(expr, markso = 1, sort = 0, debug = 0)
}
\arguments{
\item{expr}{An expression string (scalar, vector or matrix) \cr
Each element of expr must not contain multiple terms.}

\item{markso}{= 0 not to mark the special opeartors}

\item{sort}{= 1 to sort the factors according to their names}

\item{debug}{= 1 to print the intermediate result}
}
\value{
A list consisting of \cr
 # of factors (nfactors), \cr
 a vector of operators (oper), \cr
 a vector of factors (factor), \cr
 location of each oper+factor (fromto), \cr
 a matrix of operators and terms (tab) ,  \cr
 a vector of validity flag of each term (valid), \cr
 ok flag (ok=1 if the input expr is equal to the reconstructed one.)
}
\description{
Decompose a Term into Multiplicative Factors separated by
\code{ *, /,  \%*\%,  \%\@\%}.
}
\details{
It is assumed that there is just one character in between \% and \%.
\cr
The i-th factor with the preceding operator of expr is
substr( expr0, fromto[i,1], fromto[i,2] ) where
expr0 is the blank compressed expr. \cr
The prefix operator associated with the i-th factor is oper[i]. \cr\cr
The original expression can be recovered as:
\preformatted{
res <- find_factors( expr )
expr <- paste( mapply( paste, res$oper, res$factor, sep="" ), collapse="" )
}
or
\preformatted{
expr <- paste( paste( res$oper, res$factor, sep="" ), collapse="" )
}
or \cr
#' \code{
expr <- paste( apply(res$tab,1, function(x){paste(x,collapse="")} )
, collapse="")
}
\cr
The unbalanced parens, if any,  will be neglected but recorded in
\code{valid} flag.
\cr
\cr
The factors can be sorted according to their names as follows: \cr
\preformatted{
res <- find_factors("A*d/a/c", sort=1)
exprs <- paste( paste( res$oper, res$factor, sep="" ), collapse="" )

res <- find_factors("A*c/(x+y)/d", sort=1)
exprs <- paste( paste( res$oper, res$factor, sep="" ), collapse="" )
}
}
\examples{
find_factors( "a*b/c*(d-e)*f(n)" )
find_factors( c("a*b/c*(d-e)*f(n)", "-2.13*a") )
find_factors( "a\%*\%b/c\%*\%(d-e)\%*\%f(n)" )
find_factors( "ab\%*\%cd*(b\%p\%q+a\%m\%c)/c*d/a\%T\%b" )
# error
find_factors( "a+b" )

}
