% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_terms.R
\name{find_terms}
\alias{find_terms}
\title{Decompose an Expression into Additive Terms separated by \code{+} or \code{-}}
\usage{
find_terms(expr, markso = 1, sort = 0, debug = 0)
}
\arguments{
\item{expr}{An expression string (scalar, vector or matrix)}

\item{markso}{= 0 not to mark the special opeartors}

\item{sort}{= 1 to sort the factors according to their names}

\item{debug}{= 1 to print the intermediate result}
}
\value{
A list consisting of \cr
# of terms (nterms), \cr
a vector of operators (oper), \cr
a vector of terms (term), \cr
location of each oper+term (fromto), \cr
a matrix of operators and terms (tab),  \cr
a vector of validity flag of each term (valid), \cr
ok flag (ok=1 if the input expr is equal to the reconstructed one.)
}
\description{
Decompose an Expression into Additive Terms separated by \code{+} or \code{-}
}
\details{
The i-th term with the preceding operator of expr is
substr( expr0, fromto[i,1], fromto[i,2] ) where
expr0 is the blank compressed expr. \cr
The prefix operator associated with the i-th term is oper[i]. \cr\cr
The original expression can be recovered as:
\preformatted{
res <- find_terms( expr )
expr <- paste( mapply( paste, res$oper, res$term, sep="" ), collapse="" )
} or \cr
\code{
expr <- paste( paste( res$oper, res$term, sep="" ), collapse="" )
} or \cr
\code{
expr <- paste( apply(res$tab,1, function(x){paste(x,collapse="")} )
, collapse="")
}
\cr
\cr
The unbalanced parens, if any,  will be neglected but recorded in
\code{valid} flag.
\cr
\cr
The terms can be sorted according to their names as follows: \cr
\preformatted{
res <- find_terms("c-a+d-b", sort=1)
exprs <- paste( paste( res$oper, res$term, sep="" ), collapse="" )
}
}
\examples{
res=find_terms( "-((a)) - c + ((a)+(b)) + (a-b) + 1 - (x-y)*(((z+1)))" )
paste( apply(res$tab,1, function(x){paste(x,collapse="")} ), collapse="")
find_terms("-a")
find_terms("x*a/(1-y)")
find_terms("-a+b-0")
find_terms("(A\%p\%B)\%*\%C-D\%*\%diag(X) \%m\% E")
find_terms(Simplify("-a+b-0"))

find_terms("(((b)+c")
find_terms(")*c+5")


}
