% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summation.R
\name{mat2sum}
\alias{mat2sum}
\title{Express the i-j element of an Matrix Expression using Summation Operators}
\usage{
mat2sum(
  exp,
  row = "i",
  col = "j",
  root = "s",
  sublist = NA,
  simple = 1,
  expand = 1,
  print = 0,
  debug = 0
)
}
\arguments{
\item{exp}{An full symbplic matrix expression}

\item{row}{row subscript name}

\item{col}{column subscript name}

\item{root}{root of the intermediate subscripts}

\item{sublist}{a character vector containing the intermediate subscript names}

\item{simple}{= 0 to use full spec: {subscript, from , to}}

\item{expand}{= 0 not to expand (A+B) or (A*B) etc.}

\item{print}{= 1 to print the result}

\item{debug}{= 1 to print the intermediate result}
}
\description{
Express the i-j element of an Matrix Expression using Summation Operators
}
\details{
The summation of an expression over subscript k is expressed as \cr
\code{ s( exp, {k} )}  \cr
or \cr
\code{ s( exp, {k, from, to} ) }
\cr\cr
The lowercase object is treated as a column vector. (See examples below.)
\cr\cr
When row="i" and col="j": \cr
\preformatted{
 mat2sum( "A \%*\% B" ) -> s(A[i,s1]*B[s1,j], {s1})
 mat2sum( "A \%*\% B", root="z" ) -> s(A[ij,z1]*B[z1,j], {z1})
 mat2sum( "A \%*\% B", simple=0 ) -> s(A[i,s1]*B[s1,j], {s1,1,ncol(A)})
 mat2sum( "t(A) \%*\% B" ) -> s(A[s1,i]*B[s1,j], {s1})
 mat2sum( "A \%*\% t(B)" ) -> s(A[i,s1]*B[j,s1], {s1})
 mat2sum( "A \%*\%Diag(B)\%*\%C" ) -> s(A[i,s1]*B[s1,s1]*C[s1,j],{s1})
 mat2sum( "A \%*\% b" ) -> s(A[i,s1]*b[s1],{s1})
 mat2sum( "t(a) \%*\% B" ) -> s(a[s1]*B[s1,j],{s1})
 }
 \cr

 Identity matrix, I, and vec function are not recognized.

 Diag or diag function will be recognized.

 Use parentheses to mark Hadamar and Kronecker products.
 \cr
 \cr
 The resulting summation expression can be expanded and evaluated by
 sumEval function.  (lazy.symbolic2 package is required.)
}
\examples{
mat2sum( "A \%*\% B" )
mat2sum( "A \%*\% B", root="z" )
mat2sum( "A \%*\% B", simple=0 )
mat2sum( "t(A) \%*\% B" )
mat2sum( "A \%*\% t(B)" )
mat2sum( "A \%*\%Diag(B)\%*\%C" )
mat2sum( "A \%*\% b" )
mat2sum( "t(a) \%*\% B" )
mat2sum( "a \%*\% t(b)" )
mat2sum( "A\%*\%(B+C)" )
mat2sum( "A\%*\%(B+C)", expand=0 )

exp="t(A)\%*\%t(B)\%*\%C - D\%*\%t(E)"
mat2sum( exp )
mat2sum( exp, root="k" )
mat2sum( exp, sublist=letters[11:20] )
exp="A\%*\%diag(B)\%*\%C - diag(D)\%*\%E\%*\%diag(F)\%*\%G + diag(H)"
mat2sum( exp )


sexp0=mat2sum( "A\%*\%(B+C)", expand=0 )
sexp=mat2sum( "A\%*\%(B+C)" )
sexpe=Expand(c(sexp0,sexp))
Print(sexp0,sexp,sexpe)

}
